/* cryptogram puzzle tool.
 * Compile with:  cc cgram.c display.c -o cgram -lncurses
 * Copyright 2009 Jacob Mandelson
 * This code may be freely distributed under the terms of the
 * Affero General Public License version 3, in the file agpl-3.0.txt 
 * and also available from http://www.gnu.org/licenses/agpl-3.0.txt 
 */
#include "display.h"

#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <stdio.h>
#include <errno.h>
#include <ctype.h>



static void init_grid(char ***, int, int);
static void print_usage(char *);
static void read_file(FILE *, char **, int, int, int[], int);
static void update_grid(char ch1, char ch2, char **, int, int);


int main(int argc, char *argv[]) {
  int rows, cols;
  char **grid;
  FILE *f;
  int harr[26] = {0};
  char transpose[26] = {0};
  int do_hist = 0;

  if (argc == 3 && !strcmp(argv[1], "-h")) {
    do_hist = 1;
    argc--;  argv++;
  }

  if (argc != 2) {
    print_usage(argv[0]);
    exit(1);
  }

  f = fopen(argv[1], "r");
  if (f == NULL) {
    perror("fopen");
    exit(2);
  }

  init_display();
  get_winsize(&rows, &cols);
  init_grid(&grid, rows-2, cols);
  read_file(f, grid, rows-2, cols, harr, do_hist);

  draw_alphabet();

  for (;;) {
    int ch1, ch2, x;
    ch1 = read_letter();

    if (!isalpha(ch1))
      continue;
    ch1 = tolower(ch1);
    display_char_rv(ch1, rows-2, cols-26+ch1-'a');

    ch2 = read_letter();
    display_char(ch1, rows-2, cols-26+ch1-'a');

    if (ch2 == ' ') {
      display_char(ch2, rows-1, cols-26+ch1-'a');
      transpose[ch1-'a'] = '\0';
      update_grid(ch1, ch1, grid, rows-2, cols);
      continue;
    }
    if (!isalpha(ch2)) 
      continue;
    ch2 = toupper(ch2);
    display_char(ch2, rows-1, cols-26+ch1-'a');
    for (x = 0; x < 26; x++) {
      if (transpose[x] == ch2) {
        display_char(' ', rows-1, cols-26+x);
        update_grid(x+'a', x+'a', grid, rows-2, cols);
        transpose[x] = '\0';
      }
    }
    transpose[ch1-'a'] = ch2;
    update_grid(ch1, ch2, grid, rows-2, cols);
  }

  fin_display();
  return 0;
}


static void init_grid(char ***grid, int rows, int cols) {
  int i;
  *grid = malloc(rows * sizeof(char *));  
  for (i = 0; i < rows; i++) {
    (*grid)[i] = malloc(cols);
    memset((*grid)[i], ' ', cols);
  }
}

static void print_usage(char *name) {
  printf("Usage:  %s [-h] <textfile>\n", name);
}

static void read_file(FILE *f, char **grid, int rows, int cols, int harr[],
		      int do_hist) {
  int i=0, j=0;
  int ch;
  while (i < rows && (ch = getc(f)) != EOF) {
    if (ch == '\n') {
      i++; j=0;
      continue;
    } 
    if (isalpha(ch)) {
      ch = tolower(ch);
      if (do_hist)
        display_char('#', rows-(++harr[ch - 'a']), cols-26+ch-'a');
      grid[i][j] = ch;
    }
    if (isgraph(ch)) {
      display_char(ch, i, j);
    }
    if (++j == cols) {
      i++; j=0;
    }
  }
}

static void update_grid(char c1, char c2, char **grid, int rows, int cols) {
  int i, j;
  for (i = 0; i < rows; i++)
    for (j = 0; j < cols; j++) {
      if (grid[i][j] == c1)
	display_char(c2, i, j);  
    }
}
